local addon, nkUI = ...

if not Library then Library = {} end
if not Library.nkUI then Library.nkUI = {} end

nkUI.Tools = {}	

Library.nkUI.handler = {}
Library.nkUI.Event = {}

nkUI.privateVars = {}

--nkUI.privateVars.coRoutines = {}

nkUI.context = UI.CreateContext("nkUI")

function nkUICreateFrame (frameType, name, parent)

	local uiObject = nil

	if frameType == 'nkCheckbox' then
		uiObject = nkUI.nkCheckbox(name, parent)
	elseif frameType == 'nkDialog' then
		uiObject = nkUI.nkDialog(name, parent)
	elseif frameType == 'nkWindow' then
		uiObject = nkUI.nkWindow(name, parent)
	elseif frameType == 'nkCombobox' then
		uiObject = nkUI.nkCombobox(name, parent)
	elseif frameType == 'nkScrollbox' then
		uiObject = nkUI.nkScrollbox(name, parent)
	elseif frameType == 'nkSlider' then
		uiObject = nkUI.nkSlider(name, parent)
	elseif frameType == 'nkTextfield' then
		uiObject = nkUI.nkTextfield(name, parent)
	elseif frameType == 'nkTooltip' then
		uiObject = nkUI.nkTooltip(name, parent)
	end

	return uiObject

end

----------VARIOUS HELPFULL TOOLS ------------------------

function nkUI.Tools:HexToRGB(hexValue)

	if type(hexValue) ~= "string" then return nil end
	if string.len(hexValue) < 6 then return nil end
	
	local retValue = {}
	
	for i = 1, 6, 2 do		
		local subString = string.sub(hexValue, i, (i+1))
		local decValue = tonumber(subString, 16)
		table.insert (retValue, (1 / 255 * decValue))
	end
	
	return retValue
	
end

function nkUI.Tools:RGBToHex(red, green, blue)

	local redHex = nkUI.Tools:DecToHex(red)
	local greenHex = nkUI.Tools:DecToHex(green)
	local blueHex = nkUI.Tools:DecToHex(blue)
	
	local retValue = ''
	
	if red == 0 then
		retValue = '00'
	elseif string.len(redHex) == 1 then
		retValue = '0' .. redHex
	else
		retValue = redHex
	end

	if green == 0 then
		retValue = retValue .. '00'
	elseif string.len(greenHex) == 1 then
		retValue = retValue .. '0' .. greenHex
	else
		retValue = retValue .. greenHex
	end
	
	if blue == 0 then
		retValue = retValue .. '00'
	elseif string.len(blueHex) == 1 then
		retValue = retValue .. '0' .. blueHex
	else
		retValue = retValue .. blueHex
	end

	return retValue
	
end

function nkUI.Tools:DecToHex(value)

    local b,k,result,i,d=16,"0123456789ABCDEF","",0
	
    while value > 0 do
        i=i+1
        value,d = math.floor(value/b),math.mod(value,b)+1
        result=string.sub(k,d,d)..result
    end
	
    return result
	
end

function nkUI.Tools:ColorAdjust(color, adjust)

	if type(color) ~= "string" then return nil end
	if string.len(color) < 6 then return nil end
	local newColor = ''
	
	for i = 1, 6, 2 do		
		local subString = string.sub(color, i, (i+1))
		local decValue = tonumber(subString, 16)
		decValue = decValue * adjust
		local hex = nkUI.Tools:DecToHex(decValue)
		if string.len(hex) == 1 then hex = '0' .. hex end			
		newColor = newColor .. hex
	end
	
	return newColor

end

function nkUI.Tools.round (num, idp)

	if num == nil then return nil end

	local mult = 10^(idp or 0)
	return math.floor(tonumber(num) * mult + 0.5) / mult
end

function nkUI.Tools.tableCopy (tableToCopy)

    local lookup_table = {}
    local function _copy(tableToCopy)
        if type(tableToCopy) ~= "table" then
            return tableToCopy
        elseif lookup_table[tableToCopy] then
            return lookup_table[tableToCopy]
        end
        local new_table = {}
        lookup_table[tableToCopy] = new_table
        for index, value in pairs(tableToCopy) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(tableToCopy))
    end
    return _copy(tableToCopy)
end

-- function nkUI.Events.updateHandler()

	-- if #nkUI.privateVars.coRoutines > 0 then
		-- for idx = 1, #nkUI.privateVars.coRoutines, 1 do
			-- if nkUI.privateVars.coRoutines[idx].active == true then
				-- local status, value = coroutine.resume(nkUI.privateVars.coRoutines[idx].func, nkUI.privateVars.coRoutines[idx].para1, nkUI.privateVars.coRoutines[idx].para2)
				
				-- if status == false then 
					-- print ('error in coroutine: ' .. value)
					-- nkUI.privateVars.coRoutines[idx].active = false
				-- elseif value >= nkUI.privateVars.coRoutines[idx].counter or status == false then
					-- nkUI.privateVars.coRoutines[idx].active = false
				-- end
			-- end
		-- end
		
		-- if nkUI.privateVars.coRoutines[1].active == false then
			-- if nkUI.privateVars.coRoutines[1].callBack ~= nil then nkUI.privateVars.coRoutines[1].callBack() end
			-- table.remove(nkUI.privateVars.coRoutines, 1)
		-- end
	-- end	

-- end

-- --table.insert(Event.System.Update.Begin, { nkUI.Events.updateHandler, "nkUI", "updateHandler" })
-- Command.Event.Attach(Event.System.Update.Begin, nkUI.Events.updateHandler, "nkUIUpdateHandler")